@@ -29,6 +29,7 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winreg.h"
+#include "winternl.h"
 #include "ole2.h"
 
 #include "corerror.h"
@@ -39,6 +40,7 @@
 #include "metahost.h"
 #include "fusion.h"
 #include "wine/list.h"
+#include "wine/heap.h"
 #include "mscoree_private.h"
 
 #include "wine/debug.h"
@@ -86,6 +88,7 @@ typedef void (CDECL *MonoProfilerRuntimeShutdownBeginCallback) (MonoProfiler *pr
 
 MonoImage* (CDECL *mono_assembly_get_image)(MonoAssembly *assembly);
 MonoAssembly* (CDECL *mono_assembly_load_from)(MonoImage *image, const char *fname, MonoImageOpenStatus *status);
+const char* (CDECL *mono_assembly_name_get_name)(MonoAssemblyName *aname);
 MonoAssembly* (CDECL *mono_assembly_open)(const char *filename, MonoImageOpenStatus *status);
 void (CDECL *mono_callspec_set_assembly)(MonoAssembly *assembly);
 MonoClass* (CDECL *mono_class_from_mono_type)(MonoType *type);
@@ -126,8 +129,6 @@ void (CDECL *mono_thread_manage)(void);
 void (CDECL *mono_trace_set_print_handler)(MonoPrintCallback callback);
 void (CDECL *mono_trace_set_printerr_handler)(MonoPrintCallback callback);
 
-static BOOL get_mono_path(LPWSTR path);
-
 static BOOL find_mono_dll(LPCWSTR path, LPWSTR dll_path);
 
 static MonoAssembly* CDECL mono_assembly_preload_hook_fn(MonoAssemblyName *aname, char **assemblies_path, void *user_data);
@@ -193,6 +194,7 @@ static HRESULT load_mono(LPCWSTR mono_path)
 
         LOAD_MONO_FUNCTION(mono_assembly_get_image);
         LOAD_MONO_FUNCTION(mono_assembly_load_from);
+        LOAD_MONO_FUNCTION(mono_assembly_name_get_name);
         LOAD_MONO_FUNCTION(mono_assembly_open);
         LOAD_MONO_FUNCTION(mono_config_parse);
         LOAD_MONO_FUNCTION(mono_class_from_mono_type);
@@ -446,8 +448,8 @@ static HRESULT WINAPI CLRRuntimeInfo_GetVersionString(ICLRRuntimeInfo* iface,
 
 static BOOL get_install_root(LPWSTR install_dir)
 {
-    const WCHAR dotnet_key[] = {'S','O','F','T','W','A','R','E','\\','M','i','c','r','o','s','o','f','t','\\','.','N','E','T','F','r','a','m','e','w','o','r','k','\\',0};
-    const WCHAR install_root[] = {'I','n','s','t','a','l','l','R','o','o','t',0};
+    static const WCHAR dotnet_key[] = {'S','O','F','T','W','A','R','E','\\','M','i','c','r','o','s','o','f','t','\\','.','N','E','T','F','r','a','m','e','w','o','r','k','\\',0};
+    static const WCHAR install_root[] = {'I','n','s','t','a','l','l','R','o','o','t',0};
 
     DWORD len;
     HKEY key;
@@ -663,7 +665,7 @@ static BOOL find_mono_dll(LPCWSTR path, LPWSTR dll_path)
     return (attributes != INVALID_FILE_ATTRIBUTES);
 }
 
-static BOOL get_mono_path(LPWSTR path)
+static BOOL get_mono_path_local(LPWSTR path)
 {
     static const WCHAR subdir_mono[] = {'\\','m','o','n','o','\\','m','o','n','o','-','2','.','0', 0};
     WCHAR base_path[MAX_PATH], mono_dll_path[MAX_PATH];
@@ -681,6 +683,112 @@ static BOOL get_mono_path(LPWSTR path)
     return FALSE;
 }
 
+static BOOL get_mono_path_registry(LPWSTR path)
+{
+    static const WCHAR keyname[] = {'S','o','f','t','w','a','r','e','\\','W','i','n','e','\\','M','o','n','o',0};
+    static const WCHAR valuename[] = {'R','u','n','t','i','m','e','P','a','t','h',0};
+    WCHAR base_path[MAX_PATH], mono_dll_path[MAX_PATH];
+    HKEY hkey;
+    DWORD res, valuesize;
+    BOOL ret=FALSE;
+
+    /* @@ Wine registry key: HKCU\Software\Wine\Mono */
+    res = RegOpenKeyW(HKEY_CURRENT_USER, keyname, &hkey);
+    if (res != ERROR_SUCCESS)
+        return FALSE;
+
+    valuesize = sizeof(base_path);
+    res = RegGetValueW(hkey, NULL, valuename, RRF_RT_REG_SZ, NULL, base_path, &valuesize);
+    if (res == ERROR_SUCCESS && find_mono_dll(base_path, mono_dll_path))
+    {
+        strcpyW(path, base_path);
+        ret = TRUE;
+    }
+
+    RegCloseKey(hkey);
+
+    return ret;
+}
+
+static BOOL get_mono_path_unix(const char *unix_dir, LPWSTR path)
+{
+    static const WCHAR unix_prefix[] = {'\\','\\','?','\\','u','n','i','x','\\'};
+    static WCHAR * (CDECL *p_wine_get_dos_file_name)(const char*);
+    LPWSTR dos_dir;
+    WCHAR mono_dll_path[MAX_PATH];
+    BOOL ret;
+
+    if (!p_wine_get_dos_file_name)
+    {
+        p_wine_get_dos_file_name = (void*)GetProcAddress(GetModuleHandleA("kernel32"), "wine_get_dos_file_name");
+        if (!p_wine_get_dos_file_name)
+            return FALSE;
+    }
+
+    dos_dir = p_wine_get_dos_file_name(unix_dir);
+    if (!dos_dir)
+        return FALSE;
+
+    if (memcmp(dos_dir, unix_prefix, sizeof(unix_prefix)) == 0)
+    {
+        /* No drive letter for this directory */
+        heap_free(dos_dir);
+        return FALSE;
+    }
+
+    ret = find_mono_dll(dos_dir, mono_dll_path);
+    if (ret)
+        strcpyW(path, dos_dir);
+
+    heap_free(dos_dir);
+
+    return ret;
+}
+
+static BOOL get_mono_path_datadir(LPWSTR path)
+{
+    const char *data_dir;
+    char *package_dir;
+    int len;
+    BOOL ret;
+
+    if((data_dir = wine_get_data_dir()))
+    {
+        len = strlen(data_dir);
+        package_dir = heap_alloc(len + sizeof("/mono/wine-mono-" WINE_MONO_VERSION));
+        memcpy(package_dir, data_dir, len);
+        strcpy(package_dir+len, "/mono/wine-mono-" WINE_MONO_VERSION);
+    }
+    else if((data_dir = wine_get_build_dir()))
+    {
+        len = strlen(data_dir);
+        package_dir = heap_alloc(len + sizeof("/../wine-mono-" WINE_MONO_VERSION));
+        memcpy(package_dir, data_dir, len);
+        strcpy(package_dir+len, "/../wine-mono-" WINE_MONO_VERSION);
+    }
+    else
+    {
+        return FALSE;
+    }
+
+    ret = get_mono_path_unix(package_dir, path);
+
+    heap_free(package_dir);
+
+    return ret;
+}
+
+BOOL get_mono_path(LPWSTR path)
+{
+    return get_mono_path_local(path) ||
+        get_mono_path_registry(path) ||
+        get_mono_path_datadir(path) ||
+        get_mono_path_unix(INSTALL_DATADIR "/wine/mono/wine-mono-" WINE_MONO_VERSION, path) ||
+        (strcmp(INSTALL_DATADIR, "/usr/share") &&
+         get_mono_path_unix("/usr/share/wine/mono/wine-mono-" WINE_MONO_VERSION, path)) ||
+        get_mono_path_unix("/opt/wine/mono/wine-mono-" WINE_MONO_VERSION, path);
+}
+
 struct InstalledRuntimeEnum
 {
     IEnumUnknown IEnumUnknown_iface;
@@ -1126,11 +1234,10 @@ static HRESULT WINAPI metahostpolicy_GetRequestedRuntime(ICLRMetaHostPolicy *ifa
         pwzVersion, pcchVersion, pwzImageVersion, pcchImageVersion, pdwConfigFlags,
         debugstr_guid(riid), ppRuntime);
 
-    if (pCfgStream)
-        FIXME("ignoring config file stream\n");
-
-    if (pdwConfigFlags)
+    if (pdwConfigFlags) {
         FIXME("ignoring config flags\n");
+        *pdwConfigFlags = 0;
+    }
 
     if(dwPolicyFlags & METAHOST_POLICY_USE_PROCESS_IMAGE_PATH)
     {
@@ -1145,7 +1252,7 @@ static HRESULT WINAPI metahostpolicy_GetRequestedRuntime(ICLRMetaHostPolicy *ifa
     if(dwPolicyFlags & METAHOST_POLICY_APPLY_UPGRADE_POLICY)
         flags |= RUNTIME_INFO_UPGRADE_VERSION;
 
-    hr = get_runtime_info(path, pwzImageVersion, NULL, 0, flags, FALSE, &result);
+    hr = get_runtime_info(path, pwzImageVersion, NULL, pCfgStream, 0, flags, FALSE, &result);
     if (SUCCEEDED(hr))
     {
         if (pwzImageVersion)
@@ -1181,6 +1288,274 @@ HRESULT CLRMetaHostPolicy_CreateInstance(REFIID riid, void **ppobj)
     return ICLRMetaHostPolicy_QueryInterface(&GlobalCLRMetaHostPolicy.ICLRMetaHostPolicy_iface, riid, ppobj);
 }
 
+/*
+ * Assembly search override settings:
+ *
+ * WINE_MONO_OVERRIDES=*,Gac=n
+ *  Never search the GAC for libraries.
+ *
+ * WINE_MONO_OVERRIDES=Microsoft.Xna.Framework,Gac=n
+ *  Never search the GAC for Microsoft.Xna.Framework
+ *
+ * WINE_MONO_OVERRIDES=Microsoft.Xna.Framework.*,Gac=n;Microsoft.Xna.Framework.GamerServices,Gac=y
+ *  Never search the GAC for Microsoft.Xna.Framework, or any library starting
+ *  with Microsoft.Xna.Framework, except for Microsoft.Xna.Framework.GamerServices
+ */
+
+/* assembly search override flags */
+#define ASSEMBLY_SEARCH_GAC 1
+#define ASSEMBLY_SEARCH_UNDEFINED 2
+#define ASSEMBLY_SEARCH_DEFAULT ASSEMBLY_SEARCH_GAC
+
+typedef struct override_entry {
+    char *name;
+    DWORD flags;
+    struct list entry;
+} override_entry;
+
+static struct list env_overrides = LIST_INIT(env_overrides);
+
+#define IS_OPTION_TRUE(ch) \
+    ((ch) == 'y' || (ch) == 'Y' || (ch) == 't' || (ch) == 'T' || (ch) == '1')
+#define IS_OPTION_FALSE(ch) \
+    ((ch) == 'n' || (ch) == 'N' || (ch) == 'f' || (ch) == 'F' || (ch) == '0')
+
+static void parse_override_entry(override_entry *entry, const char *string, int string_len)
+{
+    const char *next_key, *equals, *value;
+    UINT kvp_len, key_len;
+
+    entry->flags = ASSEMBLY_SEARCH_DEFAULT;
+
+    while (string && string_len > 0)
+    {
+        next_key = memchr(string, ',', string_len);
+
+        if (next_key)
+        {
+            kvp_len = next_key - string;
+            next_key++;
+        }
+        else
+            kvp_len = string_len;
+
+        equals = memchr(string, '=', kvp_len);
+
+        if (equals)
+        {
+            key_len = equals - string;
+            value = equals + 1;
+            switch (key_len) {
+            case 3:
+                if (!_strnicmp(string, "gac", 3)) {
+                    if (IS_OPTION_TRUE(*value))
+                        entry->flags |= ASSEMBLY_SEARCH_GAC;
+                    else if (IS_OPTION_FALSE(*value))
+                        entry->flags &= ~ASSEMBLY_SEARCH_GAC;
+                }
+                break;
+            default:
+                break;
+            }
+        }
+
+        string = next_key;
+        string_len -= kvp_len + 1;
+    }
+}
+
+static BOOL WINAPI parse_env_overrides(INIT_ONCE *once, void *param, void **context)
+{
+    const char *override_string = getenv("WINE_MONO_OVERRIDES");
+    struct override_entry *entry;
+
+    if (override_string)
+    {
+        const char *entry_start;
+
+        entry_start = override_string;
+
+        while (entry_start && *entry_start)
+        {
+            const char *next_entry, *basename_end;
+            UINT entry_len;
+
+            next_entry = strchr(entry_start, ';');
+
+            if (next_entry)
+            {
+                entry_len = next_entry - entry_start;
+                next_entry++;
+            }
+            else
+                entry_len = strlen(entry_start);
+
+            basename_end = memchr(entry_start, ',', entry_len);
+
+            if (!basename_end)
+            {
+                entry_start = next_entry;
+                continue;
+            }
+
+            entry = heap_alloc_zero(sizeof(*entry));
+            if (!entry)
+            {
+                ERR("out of memory\n");
+                break;
+            }
+
+            entry->name = heap_alloc_zero(basename_end - entry_start + 1);
+            if (!entry->name)
+            {
+                ERR("out of memory\n");
+                heap_free(entry);
+                break;
+            }
+
+            memcpy(entry->name, entry_start, basename_end - entry_start);
+
+            entry_len -= basename_end - entry_start + 1;
+            entry_start = basename_end + 1;
+
+            parse_override_entry(entry, entry_start, entry_len);
+
+            list_add_tail(&env_overrides, &entry->entry);
+
+            entry_start = next_entry;
+        }
+    }
+
+    return TRUE;
+}
+
+static DWORD get_basename_search_flags(const char *basename, MonoAssemblyName *aname, HKEY userkey, HKEY appkey)
+{
+    struct override_entry *entry;
+    char buffer[256];
+    DWORD buffer_size;
+    static INIT_ONCE init_once = INIT_ONCE_STATIC_INIT;
+
+    InitOnceExecuteOnce(&init_once, parse_env_overrides, NULL, NULL);
+
+    LIST_FOR_EACH_ENTRY(entry, &env_overrides, override_entry, entry)
+    {
+        if (strcmp(basename, entry->name) == 0)
+        {
+            return entry->flags;
+        }
+    }
+
+    buffer_size = sizeof(buffer);
+    if (appkey && !RegQueryValueExA(appkey, basename, 0, NULL, (LPBYTE)buffer, &buffer_size))
+    {
+        override_entry reg_entry;
+
+        memset(&reg_entry, 0, sizeof(reg_entry));
+
+        parse_override_entry(&reg_entry, buffer, strlen(buffer));
+
+        return reg_entry.flags;
+    }
+
+    buffer_size = sizeof(buffer);
+    if (userkey && !RegQueryValueExA(userkey, basename, 0, NULL, (LPBYTE)buffer, &buffer_size))
+    {
+        override_entry reg_entry;
+
+        memset(&reg_entry, 0, sizeof(reg_entry));
+
+        parse_override_entry(&reg_entry, buffer, strlen(buffer));
+
+        return reg_entry.flags;
+    }
+
+    return ASSEMBLY_SEARCH_UNDEFINED;
+}
+
+static HKEY get_app_overrides_key(void)
+{
+    static const WCHAR subkeyW[] = {'\\','M','o','n','o','\\','A','s','m','O','v','e','r','r','i','d','e','s',0};
+    WCHAR bufferW[MAX_PATH+18];
+    HKEY appkey = 0;
+    DWORD len;
+
+    len = (GetModuleFileNameW( 0, bufferW, MAX_PATH ));
+    if (len && len < MAX_PATH)
+    {
+        HKEY tmpkey;
+        WCHAR *p, *appname = bufferW;
+        if ((p = strrchrW( appname, '/' ))) appname = p + 1;
+        if ((p = strrchrW( appname, '\\' ))) appname = p + 1;
+        strcatW( appname, subkeyW );
+        /* @@ Wine registry key: HKCU\Software\Wine\AppDefaults\app.exe\Mono\AsmOverrides */
+        if (!RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\AppDefaults", &tmpkey ))
+        {
+            if (RegOpenKeyW( tmpkey, appname, &appkey )) appkey = 0;
+            RegCloseKey( tmpkey );
+        }
+    }
+
+    return appkey;
+}
+
+static DWORD get_assembly_search_flags(MonoAssemblyName *aname)
+{
+    const char *name = mono_assembly_name_get_name(aname);
+    char *name_copy, *name_end;
+    DWORD result;
+    HKEY appkey = 0, userkey;
+
+    /* @@ Wine registry key: HKCU\Software\Wine\Mono\AsmOverrides */
+    if (RegOpenKeyA( HKEY_CURRENT_USER, "Software\\Wine\\Mono\\AsmOverrides", &userkey )) userkey = 0;
+
+    appkey = get_app_overrides_key();
+
+    result = get_basename_search_flags(name, aname, userkey, appkey);
+    if (result != ASSEMBLY_SEARCH_UNDEFINED)
+    {
+        if (userkey) RegCloseKey(userkey);
+        if (appkey) RegCloseKey(appkey);
+        return result;
+    }
+
+    name_copy = heap_alloc((strlen(name) + 3) * sizeof(WCHAR));
+    if (!name_copy)
+    {
+        ERR("out of memory\n");
+        if (userkey) RegCloseKey(userkey);
+        if (appkey) RegCloseKey(appkey);
+        return ASSEMBLY_SEARCH_DEFAULT;
+    }
+
+    strcpy(name_copy, name);
+    name_end = name_copy + strlen(name);
+
+    do
+    {
+        strcpy(name_end, ".*");
+        result = get_basename_search_flags(name_copy, aname, userkey, appkey);
+        if (result != ASSEMBLY_SEARCH_UNDEFINED) break;
+
+        *name_end = 0;
+        name_end = strrchr(name_copy, '.');
+    } while (name_end != NULL);
+
+    /* default flags */
+    if (result == ASSEMBLY_SEARCH_UNDEFINED)
+    {
+        result = get_basename_search_flags("*", aname, userkey, appkey);
+        if (result == ASSEMBLY_SEARCH_UNDEFINED)
+            result = ASSEMBLY_SEARCH_DEFAULT;
+    }
+
+    heap_free(name_copy);
+    if (appkey) RegCloseKey(appkey);
+    if (userkey) RegCloseKey(userkey);
+
+    return result;
+}
+
 HRESULT get_file_from_strongname(WCHAR* stringnameW, WCHAR* assemblies_path, int path_length)
 {
     HRESULT hr=S_OK;
@@ -1230,6 +1605,7 @@ static MonoAssembly* CDECL mono_assembly_preload_hook_fn(MonoAssemblyName *aname
     WCHAR path[MAX_PATH];
     char *pathA;
     MonoImageOpenStatus stat;
+    DWORD search_flags;
 
     stringname = mono_stringify_assembly_name(aname);
 
@@ -1237,38 +1613,45 @@ static MonoAssembly* CDECL mono_assembly_preload_hook_fn(MonoAssemblyName *aname
 
     if (!stringname) return NULL;
 
+    search_flags = get_assembly_search_flags(aname);
+
     /* FIXME: We should search the given paths before the GAC. */
 
-    stringnameW_size = MultiByteToWideChar(CP_UTF8, 0, stringname, -1, NULL, 0);
-
-    stringnameW = HeapAlloc(GetProcessHeap(), 0, stringnameW_size * sizeof(WCHAR));
-    if (stringnameW)
+    if ((search_flags & ASSEMBLY_SEARCH_GAC) != 0)
     {
-        MultiByteToWideChar(CP_UTF8, 0, stringname, -1, stringnameW, stringnameW_size);
+        stringnameW_size = MultiByteToWideChar(CP_UTF8, 0, stringname, -1, NULL, 0);
 
-        hr = get_file_from_strongname(stringnameW, path, MAX_PATH);
-
-        HeapFree(GetProcessHeap(), 0, stringnameW);
-    }
-    else
-        hr = E_OUTOFMEMORY;
-
-    if (SUCCEEDED(hr))
-    {
-        TRACE("found: %s\n", debugstr_w(path));
-
-        pathA = WtoA(path);
-
-        if (pathA)
+        stringnameW = HeapAlloc(GetProcessHeap(), 0, stringnameW_size * sizeof(WCHAR));
+        if (stringnameW)
         {
-            result = mono_assembly_open(pathA, &stat);
+            MultiByteToWideChar(CP_UTF8, 0, stringname, -1, stringnameW, stringnameW_size);
 
-            if (!result)
-                ERR("Failed to load %s, status=%u\n", debugstr_w(path), stat);
+            hr = get_file_from_strongname(stringnameW, path, MAX_PATH);
 
-            HeapFree(GetProcessHeap(), 0, pathA);
+            HeapFree(GetProcessHeap(), 0, stringnameW);
+        }
+        else
+            hr = E_OUTOFMEMORY;
+
+        if (SUCCEEDED(hr))
+        {
+            TRACE("found: %s\n", debugstr_w(path));
+
+            pathA = WtoA(path);
+
+            if (pathA)
+            {
+                result = mono_assembly_open(pathA, &stat);
+
+                if (!result)
+                    ERR("Failed to load %s, status=%u\n", debugstr_w(path), stat);
+
+                HeapFree(GetProcessHeap(), 0, pathA);
+            }
         }
     }
+    else
+        TRACE("skipping Windows GAC search due to override setting\n");
 
     mono_free(stringname);
 
@@ -1276,7 +1659,8 @@ static MonoAssembly* CDECL mono_assembly_preload_hook_fn(MonoAssemblyName *aname
 }
 
 HRESULT get_runtime_info(LPCWSTR exefile, LPCWSTR version, LPCWSTR config_file,
-    DWORD startup_flags, DWORD runtimeinfo_flags, BOOL legacy, ICLRRuntimeInfo **result)
+    IStream *config_stream, DWORD startup_flags, DWORD runtimeinfo_flags,
+    BOOL legacy, ICLRRuntimeInfo **result)
 {
     static const WCHAR dotconfig[] = {'.','c','o','n','f','i','g',0};
     static const DWORD supported_startup_flags = 0;
@@ -1297,7 +1681,7 @@ HRESULT get_runtime_info(LPCWSTR exefile, LPCWSTR version, LPCWSTR config_file,
     if (exefile && !exefile[0])
         exefile = NULL;
 
-    if (exefile && !config_file)
+    if (exefile && !config_file && !config_stream)
     {
         strcpyW(local_config_file, exefile);
         strcatW(local_config_file, dotconfig);
@@ -1305,10 +1689,13 @@ HRESULT get_runtime_info(LPCWSTR exefile, LPCWSTR version, LPCWSTR config_file,
         config_file = local_config_file;
     }
 
-    if (config_file)
+    if (config_file || config_stream)
     {
         BOOL found = FALSE;
-        hr = parse_config_file(config_file, &parsed_config);
+        if (config_file)
+            hr = parse_config_file(config_file, &parsed_config);
+        else
+            hr = parse_config_stream(config_stream, &parsed_config);
 
         if (SUCCEEDED(hr))
         {
