@@ -1227,8 +1227,14 @@ static HRESULT WINAPI HTMLDOMNode3_get_localName(IHTMLDOMNode3 *iface, VARIANT *
 static HRESULT WINAPI HTMLDOMNode3_get_namespaceURI(IHTMLDOMNode3 *iface, VARIANT *p)
 {
     HTMLDOMNode *This = impl_from_IHTMLDOMNode3(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+    nsAString nsstr;
+    nsresult nsres;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    nsAString_InitDepend(&nsstr, NULL);
+    nsres = nsIDOMNode_GetNamespaceURI(This->nsnode, &nsstr);
+    return return_nsstr_variant(nsres, &nsstr, p);
 }
 
 static HRESULT WINAPI HTMLDOMNode3_put_textContent(IHTMLDOMNode3 *iface, VARIANT v)
@@ -1573,9 +1579,9 @@ static nsresult NSAPI HTMLDOMNode_unlink(void *p)
     }
 
     if(This->doc && &This->doc->node != This) {
-        HTMLDocument *doc = &This->doc->basedoc;
+        HTMLDocumentNode *doc = This->doc;
         This->doc = NULL;
-        htmldoc_release(doc);
+        htmldoc_release(&doc->basedoc);
     }else {
         This->doc = NULL;
     }
