@@ -81,7 +81,7 @@ void msi_parse_version_string(LPCWSTR verStr, PDWORD ms, PDWORD ls)
  * Returns ERROR_SUCCESS upon success (where not finding the record counts as
  * success), something else on error.
  */
-static UINT ACTION_AppSearchGetSignature(MSIPACKAGE *package, MSISIGNATURE *sig, LPCWSTR name)
+static UINT get_signature( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *name )
 {
     static const WCHAR query[] = {
         's','e','l','e','c','t',' ','*',' ',
@@ -155,16 +155,16 @@ static UINT ACTION_AppSearchGetSignature(MSIPACKAGE *package, MSISIGNATURE *sig,
 }
 
 /* Frees any memory allocated in sig */
-static void ACTION_FreeSignature(MSISIGNATURE *sig)
+static void free_signature( MSISIGNATURE *sig )
 {
     msi_free(sig->File);
     msi_free(sig->Languages);
 }
 
-static LPWSTR app_search_file(LPWSTR path, MSISIGNATURE *sig)
+static WCHAR *search_file( MSIPACKAGE *package, WCHAR *path, MSISIGNATURE *sig )
 {
     VS_FIXEDFILEINFO *info;
-    DWORD attr, handle, size;
+    DWORD attr, size;
     LPWSTR val = NULL;
     LPBYTE buffer;
 
@@ -173,20 +173,18 @@ static LPWSTR app_search_file(LPWSTR path, MSISIGNATURE *sig)
         PathRemoveFileSpecW(path);
         PathAddBackslashW(path);
 
-        attr = GetFileAttributesW(path);
-        if (attr != INVALID_FILE_ATTRIBUTES &&
-            (attr & FILE_ATTRIBUTE_DIRECTORY))
+        attr = msi_get_file_attributes( package, path );
+        if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY))
             return strdupW(path);
 
         return NULL;
     }
 
-    attr = GetFileAttributesW(path);
-    if (attr == INVALID_FILE_ATTRIBUTES ||
-        (attr & FILE_ATTRIBUTE_DIRECTORY))
+    attr = msi_get_file_attributes( package, path );
+    if (attr == INVALID_FILE_ATTRIBUTES || (attr & FILE_ATTRIBUTE_DIRECTORY))
         return NULL;
 
-    size = GetFileVersionInfoSizeW(path, &handle);
+    size = msi_get_file_version_info( package, path, 0, NULL );
     if (!size)
         return strdupW(path);
 
@@ -194,7 +192,8 @@ static LPWSTR app_search_file(LPWSTR path, MSISIGNATURE *sig)
     if (!buffer)
         return NULL;
 
-    if (!GetFileVersionInfoW(path, 0, size, buffer))
+    size = msi_get_file_version_info( package, path, size, buffer );
+    if (!size)
         goto done;
 
     if (!VerQueryValueW(buffer, szBackSlash, (LPVOID)&info, &size) || !info)
@@ -227,7 +226,7 @@ done:
     return val;
 }
 
-static UINT ACTION_AppSearchComponents(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNATURE *sig)
+static UINT search_components( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
     static const WCHAR query[] =  {
         'S','E','L','E','C','T',' ','*',' ',
@@ -240,7 +239,6 @@ static UINT ACTION_AppSearchComponents(MSIPACKAGE *package, LPWSTR *appValue, MS
         '`','S','i','g','n','a','t','u','r','e','`',' ',
         'W','H','E','R','E',' ','`','S','i','g','n','a','t','u','r','e','`',' ','=',' ',
         '\'','%','s','\'',0};
-
     MSIRECORD *row, *rec;
     LPCWSTR signature, guid;
     BOOL sigpresent = TRUE;
@@ -275,7 +273,7 @@ static UINT ACTION_AppSearchComponents(MSIPACKAGE *package, LPWSTR *appValue, MS
     if (!*path)
         goto done;
 
-    attr = GetFileAttributesW(path);
+    attr = msi_get_file_attributes( package, path );
     if (attr == INVALID_FILE_ATTRIBUTES)
         goto done;
 
@@ -283,7 +281,7 @@ static UINT ACTION_AppSearchComponents(MSIPACKAGE *package, LPWSTR *appValue, MS
 
     if (type != msidbLocatorTypeDirectory && sigpresent && !isdir)
     {
-        *appValue = app_search_file(path, sig);
+        *appValue = search_file( package, path, sig );
     }
     else if (!sigpresent && (type != msidbLocatorTypeDirectory || isdir))
     {
@@ -302,9 +300,8 @@ static UINT ACTION_AppSearchComponents(MSIPACKAGE *package, LPWSTR *appValue, MS
         PathAddBackslashW(path);
         lstrcatW(path, MSI_RecordGetString(rec, 2));
 
-        attr = GetFileAttributesW(path);
-        if (attr != INVALID_FILE_ATTRIBUTES &&
-            !(attr & FILE_ATTRIBUTE_DIRECTORY))
+        attr = msi_get_file_attributes( package, path );
+        if (attr != INVALID_FILE_ATTRIBUTES && !(attr & FILE_ATTRIBUTE_DIRECTORY))
             *appValue = strdupW(path);
     }
 
@@ -314,8 +311,7 @@ done:
     return ERROR_SUCCESS;
 }
 
-static void ACTION_ConvertRegValue(DWORD regType, const BYTE *value, DWORD sz,
- LPWSTR *appValue)
+static void convert_reg_value( DWORD regType, const BYTE *value, DWORD sz, WCHAR **appValue )
 {
     static const WCHAR dwordFmt[] = { '#','%','d','\0' };
     static const WCHAR binPre[] = { '#','x','\0' };
@@ -365,10 +361,9 @@ static void ACTION_ConvertRegValue(DWORD regType, const BYTE *value, DWORD sz,
     }
 }
 
-static UINT ACTION_SearchDirectory(MSIPACKAGE *package, MSISIGNATURE *sig,
- LPCWSTR path, int depth, LPWSTR *appValue);
+static UINT search_directory( MSIPACKAGE *, MSISIGNATURE *, const WCHAR *, int, WCHAR ** );
 
-static UINT ACTION_AppSearchReg(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNATURE *sig)
+static UINT search_reg( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
     static const WCHAR query[] =  {
         'S','E','L','E','C','T',' ','*',' ','F','R','O','M',' ',
@@ -473,13 +468,13 @@ static UINT ACTION_AppSearchReg(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNAT
     switch (type & 0x0f)
     {
     case msidbLocatorTypeDirectory:
-        ACTION_SearchDirectory(package, sig, ptr, 0, appValue);
+        search_directory( package, sig, ptr, 0, appValue );
         break;
     case msidbLocatorTypeFileName:
-        *appValue = app_search_file(ptr, sig);
+        *appValue = search_file( package, ptr, sig );
         break;
     case msidbLocatorTypeRawValue:
-        ACTION_ConvertRegValue(regType, value, sz, appValue);
+        convert_reg_value( regType, value, sz, appValue );
         break;
     default:
         FIXME("unimplemented for type %d (key path %s, value %s)\n",
@@ -520,8 +515,7 @@ static LPWSTR get_ini_field(LPWSTR buf, int field)
     return strdupW(beg);
 }
 
-static UINT ACTION_AppSearchIni(MSIPACKAGE *package, LPWSTR *appValue,
- MSISIGNATURE *sig)
+static UINT search_ini( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
     static const WCHAR query[] =  {
         's','e','l','e','c','t',' ','*',' ',
@@ -561,10 +555,10 @@ static UINT ACTION_AppSearchIni(MSIPACKAGE *package, LPWSTR *appValue,
         switch (type & 0x0f)
         {
         case msidbLocatorTypeDirectory:
-            ACTION_SearchDirectory(package, sig, buf, 0, appValue);
+            search_directory( package, sig, buf, 0, appValue );
             break;
         case msidbLocatorTypeFileName:
-            *appValue = app_search_file(buf, sig);
+            *appValue = search_file( package, buf, sig );
             break;
         case msidbLocatorTypeRawValue:
             *appValue = get_ini_field(buf, field);
@@ -590,8 +584,7 @@ static UINT ACTION_AppSearchIni(MSIPACKAGE *package, LPWSTR *appValue,
  * - what does AppSearch return if the table values are invalid?
  * - what if dst is too small?
  */
-static void ACTION_ExpandAnyPath(MSIPACKAGE *package, WCHAR *src, WCHAR *dst,
- size_t len)
+static void expand_any_path( MSIPACKAGE *package, WCHAR *src, WCHAR *dst, size_t len )
 {
     WCHAR *ptr, *deformatted;
 
@@ -685,20 +678,20 @@ done:
  * Return ERROR_SUCCESS in case of success (whether or not the file matches),
  * something else if an install-halting error occurs.
  */
-static UINT ACTION_FileVersionMatches(const MSISIGNATURE *sig, LPCWSTR filePath,
- BOOL *matches)
+static UINT file_version_matches( MSIPACKAGE *package, const MSISIGNATURE *sig, const WCHAR *filePath,
+                                  BOOL *matches )
 {
     UINT len;
     void *version;
     VS_FIXEDFILEINFO *info = NULL;
-    DWORD zero, size = GetFileVersionInfoSizeW( filePath, &zero );
+    DWORD size = msi_get_file_version_info( package, filePath, 0, NULL );
 
     *matches = FALSE;
 
     if (!size) return ERROR_SUCCESS;
     if (!(version = msi_alloc( size ))) return ERROR_OUTOFMEMORY;
 
-    if (GetFileVersionInfoW( filePath, 0, size, version ))
+    if (msi_get_file_version_info( package, filePath, size, version ))
         VerQueryValueW( version, szBackSlash, (void **)&info, &len );
 
     if (info)
@@ -745,8 +738,8 @@ static UINT ACTION_FileVersionMatches(const MSISIGNATURE *sig, LPCWSTR filePath,
  * Return ERROR_SUCCESS in case of success (whether or not the file matches),
  * something else if an install-halting error occurs.
  */
-static UINT ACTION_FileMatchesSig(const MSISIGNATURE *sig,
- const WIN32_FIND_DATAW *findData, LPCWSTR fullFilePath, BOOL *matches)
+static UINT file_matches_sig( MSIPACKAGE *package, const MSISIGNATURE *sig, const WIN32_FIND_DATAW *findData,
+                              const WCHAR *fullFilePath, BOOL *matches )
 {
     UINT rc = ERROR_SUCCESS;
 
@@ -778,7 +771,7 @@ static UINT ACTION_FileMatchesSig(const MSISIGNATURE *sig,
         *matches = FALSE;
     if (*matches && (sig->MinVersionMS || sig->MinVersionLS ||
      sig->MaxVersionMS || sig->MaxVersionLS))
-        rc = ACTION_FileVersionMatches(sig, fullFilePath, matches);
+        rc = file_version_matches( package, sig, fullFilePath, matches );
     return rc;
 }
 
@@ -790,9 +783,10 @@ static UINT ACTION_FileMatchesSig(const MSISIGNATURE *sig,
  * Returns ERROR_SUCCESS on success (which may include non-critical errors),
  * something else on failures which should halt the install.
  */
-static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
- MSISIGNATURE *sig, LPCWSTR dir, int depth)
+static UINT recurse_search_directory( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig, const WCHAR *dir,
+                                      int depth )
 {
+    static const WCHAR starDotStarW[] = { '*','.','*',0 };
     HANDLE hFind;
     WIN32_FIND_DATAW findData;
     UINT rc = ERROR_SUCCESS;
@@ -801,10 +795,7 @@ static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
     WCHAR *buf;
     DWORD len;
 
-    static const WCHAR starDotStarW[] = { '*','.','*',0 };
-
-    TRACE("Searching directory %s for file %s, depth %d\n", debugstr_w(dir),
-          debugstr_w(sig->File), depth);
+    TRACE("Searching directory %s for file %s, depth %d\n", debugstr_w(dir), debugstr_w(sig->File), depth);
 
     if (depth < 0)
         return ERROR_SUCCESS;
@@ -823,14 +814,14 @@ static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
     PathAddBackslashW(buf);
     lstrcatW(buf, sig->File);
 
-    hFind = FindFirstFileW(buf, &findData);
+    hFind = msi_find_first_file( package, buf, &findData );
     if (hFind != INVALID_HANDLE_VALUE)
     {
         if (!(findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
         {
             BOOL matches;
 
-            rc = ACTION_FileMatchesSig(sig, &findData, buf, &matches);
+            rc = file_matches_sig( package, sig, &findData, buf, &matches );
             if (rc == ERROR_SUCCESS && matches)
             {
                 TRACE("found file, returning %s\n", debugstr_w(buf));
@@ -846,7 +837,7 @@ static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
         PathAddBackslashW(buf);
         lstrcatW(buf, starDotStarW);
 
-        hFind = FindFirstFileW(buf, &findData);
+        hFind = msi_find_first_file( package, buf, &findData );
         if (hFind != INVALID_HANDLE_VALUE)
         {
             if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY &&
@@ -855,12 +846,10 @@ static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
             {
                 lstrcpyW(subpath, dir);
                 PathAppendW(subpath, findData.cFileName);
-                rc = ACTION_RecurseSearchDirectory(package, appValue, sig,
-                                                   subpath, depth - 1);
+                rc = recurse_search_directory( package, appValue, sig, subpath, depth - 1 );
             }
 
-            while (rc == ERROR_SUCCESS && !*appValue &&
-                   FindNextFileW(hFind, &findData) != 0)
+            while (rc == ERROR_SUCCESS && !*appValue && msi_find_next_file( package, hFind, &findData ))
             {
                 if (!strcmpW( findData.cFileName, szDot ) ||
                     !strcmpW( findData.cFileName, szDotDot ))
@@ -869,8 +858,7 @@ static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
                 lstrcpyW(subpath, dir);
                 PathAppendW(subpath, findData.cFileName);
                 if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
-                    rc = ACTION_RecurseSearchDirectory(package, appValue,
-                                                       sig, subpath, depth - 1);
+                    rc = recurse_search_directory( package, appValue, sig, subpath, depth - 1 );
             }
 
             FindClose(hFind);
@@ -883,10 +871,9 @@ static UINT ACTION_RecurseSearchDirectory(MSIPACKAGE *package, LPWSTR *appValue,
     return rc;
 }
 
-static UINT ACTION_CheckDirectory(MSIPACKAGE *package, LPCWSTR dir,
- LPWSTR *appValue)
+static UINT check_directory( MSIPACKAGE *package, const WCHAR *dir, WCHAR **appValue )
 {
-    DWORD attr = GetFileAttributesW(dir);
+    DWORD attr = msi_get_file_attributes( package, dir );
 
     if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY))
     {
@@ -897,7 +884,7 @@ static UINT ACTION_CheckDirectory(MSIPACKAGE *package, LPCWSTR dir,
     return ERROR_SUCCESS;
 }
 
-static BOOL ACTION_IsFullPath(LPCWSTR path)
+static BOOL is_full_path( const WCHAR *path )
 {
     WCHAR first = toupperW(path[0]);
     BOOL ret;
@@ -911,26 +898,24 @@ static BOOL ACTION_IsFullPath(LPCWSTR path)
     return ret;
 }
 
-static UINT ACTION_SearchDirectory(MSIPACKAGE *package, MSISIGNATURE *sig,
- LPCWSTR path, int depth, LPWSTR *appValue)
+static UINT search_directory( MSIPACKAGE *package, MSISIGNATURE *sig, const WCHAR *path, int depth, WCHAR **appValue )
 {
     UINT rc;
     DWORD attr;
     LPWSTR val = NULL;
 
-    TRACE("%p, %p, %s, %d, %p\n", package, sig, debugstr_w(path), depth,
-     appValue);
+    TRACE("%p, %p, %s, %d, %p\n", package, sig, debugstr_w(path), depth, appValue);
 
-    if (ACTION_IsFullPath(path))
+    if (is_full_path( path ))
     {
         if (sig->File)
-            rc = ACTION_RecurseSearchDirectory(package, &val, sig, path, depth);
+            rc = recurse_search_directory( package, &val, sig, path, depth );
         else
         {
             /* Recursively searching a directory makes no sense when the
              * directory to search is the thing you're trying to find.
              */
-            rc = ACTION_CheckDirectory(package, path, &val);
+            rc = check_directory( package, path, &val );
         }
     }
     else
@@ -952,16 +937,14 @@ static UINT ACTION_SearchDirectory(MSIPACKAGE *package, MSISIGNATURE *sig,
             lstrcpynW(pathWithDrive + 3, path, ARRAY_SIZE(pathWithDrive) - 3);
 
             if (sig->File)
-                rc = ACTION_RecurseSearchDirectory(package, &val, sig,
-                                                   pathWithDrive, depth);
+                rc = recurse_search_directory( package, &val, sig, pathWithDrive, depth );
             else
-                rc = ACTION_CheckDirectory(package, pathWithDrive, &val);
+                rc = check_directory( package, pathWithDrive, &val );
         }
     }
 
-    attr = GetFileAttributesW(val);
-    if (attr != INVALID_FILE_ATTRIBUTES &&
-        (attr & FILE_ATTRIBUTE_DIRECTORY) &&
+    attr = msi_get_file_attributes( package, val );
+    if (attr != INVALID_FILE_ATTRIBUTES && (attr & FILE_ATTRIBUTE_DIRECTORY) &&
         val && val[lstrlenW(val) - 1] != '\\')
     {
         val = msi_realloc(val, (lstrlenW(val) + 2) * sizeof(WCHAR));
@@ -977,10 +960,9 @@ static UINT ACTION_SearchDirectory(MSIPACKAGE *package, MSISIGNATURE *sig,
     return rc;
 }
 
-static UINT ACTION_AppSearchSigName(MSIPACKAGE *package, LPCWSTR sigName,
- MSISIGNATURE *sig, LPWSTR *appValue);
+static UINT search_sig_name( MSIPACKAGE *, const WCHAR *, MSISIGNATURE *, WCHAR ** );
 
-static UINT ACTION_AppSearchDr(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNATURE *sig)
+static UINT search_dr( MSIPACKAGE *package, WCHAR **appValue, MSISIGNATURE *sig )
 {
     static const WCHAR query[] =  {
         's','e','l','e','c','t',' ','*',' ',
@@ -1014,8 +996,8 @@ static UINT ACTION_AppSearchDr(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNATU
     {
         MSISIGNATURE parentSig;
 
-        ACTION_AppSearchSigName(package, parentName, &parentSig, &parent);
-        ACTION_FreeSignature(&parentSig);
+        search_sig_name( package, parentName, &parentSig, &parent );
+        free_signature( &parentSig );
         if (!parent)
         {
             msiobj_release(&row->hdr);
@@ -1032,13 +1014,13 @@ static UINT ACTION_AppSearchDr(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNATU
         depth = MSI_RecordGetInteger(row,4);
 
     if (sz)
-        ACTION_ExpandAnyPath(package, path, expanded, MAX_PATH);
+        expand_any_path( package, path, expanded, MAX_PATH );
     else
         strcpyW(expanded, path);
 
     if (parent)
     {
-        attr = GetFileAttributesW(parent);
+        attr = msi_get_file_attributes( package, parent );
         if (attr != INVALID_FILE_ATTRIBUTES &&
             !(attr & FILE_ATTRIBUTE_DIRECTORY))
         {
@@ -1049,45 +1031,42 @@ static UINT ACTION_AppSearchDr(MSIPACKAGE *package, LPWSTR *appValue, MSISIGNATU
         strcpyW(path, parent);
         strcatW(path, expanded);
     }
-    else if (sz)
-        strcpyW(path, expanded);
+    else if (sz) strcpyW(path, expanded);
 
     PathAddBackslashW(path);
 
-    rc = ACTION_SearchDirectory(package, sig, path, depth, appValue);
+    rc = search_directory( package, sig, path, depth, appValue );
 
     msi_free(parent);
     msiobj_release(&row->hdr);
-
     TRACE("returning %d\n", rc);
     return rc;
 }
 
-static UINT ACTION_AppSearchSigName(MSIPACKAGE *package, LPCWSTR sigName,
- MSISIGNATURE *sig, LPWSTR *appValue)
+static UINT search_sig_name( MSIPACKAGE *package, const WCHAR *sigName, MSISIGNATURE *sig, WCHAR **appValue )
 {
     UINT rc;
 
     *appValue = NULL;
-    rc = ACTION_AppSearchGetSignature(package, sig, sigName);
+    rc = get_signature( package, sig, sigName );
     if (rc == ERROR_SUCCESS)
     {
-        rc = ACTION_AppSearchComponents(package, appValue, sig);
+        rc = search_components( package, appValue, sig );
         if (rc == ERROR_SUCCESS && !*appValue)
         {
-            rc = ACTION_AppSearchReg(package, appValue, sig);
+            rc = search_reg( package, appValue, sig );
             if (rc == ERROR_SUCCESS && !*appValue)
             {
-                rc = ACTION_AppSearchIni(package, appValue, sig);
+                rc = search_ini( package, appValue, sig );
                 if (rc == ERROR_SUCCESS && !*appValue)
-                    rc = ACTION_AppSearchDr(package, appValue, sig);
+                    rc = search_dr( package, appValue, sig );
             }
         }
     }
     return rc;
 }
 
-static UINT iterate_appsearch(MSIRECORD *row, LPVOID param)
+static UINT ITERATE_AppSearch(MSIRECORD *row, LPVOID param)
 {
     MSIPACKAGE *package = param;
     LPCWSTR propName, sigName;
@@ -1102,7 +1081,7 @@ static UINT iterate_appsearch(MSIRECORD *row, LPVOID param)
 
     TRACE("%s %s\n", debugstr_w(propName), debugstr_w(sigName));
 
-    r = ACTION_AppSearchSigName(package, sigName, &sig, &value);
+    r = search_sig_name( package, sigName, &sig, &value );
     if (value)
     {
         r = msi_set_property( package->db, propName, value, -1 );
@@ -1111,7 +1090,7 @@ static UINT iterate_appsearch(MSIRECORD *row, LPVOID param)
 
         msi_free(value);
     }
-    ACTION_FreeSignature(&sig);
+    free_signature( &sig );
 
     uirow = MSI_CreateRecord( 2 );
     MSI_RecordSetStringW( uirow, 1, propName );
@@ -1142,7 +1121,7 @@ UINT ACTION_AppSearch(MSIPACKAGE *package)
     if (r != ERROR_SUCCESS)
         return ERROR_SUCCESS;
 
-    r = MSI_IterateRecords( view, NULL, iterate_appsearch, package );
+    r = MSI_IterateRecords( view, NULL, ITERATE_AppSearch, package );
     msiobj_release( &view->hdr );
     return r;
 }
@@ -1161,7 +1140,7 @@ static UINT ITERATE_CCPSearch(MSIRECORD *row, LPVOID param)
 
     TRACE("%s\n", debugstr_w(signature));
 
-    ACTION_AppSearchSigName(package, signature, &sig, &value);
+    search_sig_name( package, signature, &sig, &value );
     if (value)
     {
         TRACE("Found signature %s\n", debugstr_w(signature));
@@ -1170,8 +1149,7 @@ static UINT ITERATE_CCPSearch(MSIRECORD *row, LPVOID param)
         r = ERROR_NO_MORE_ITEMS;
     }
 
-    ACTION_FreeSignature(&sig);
-
+    free_signature(&sig);
     return r;
 }
 
