@@ -44,6 +44,7 @@
 
 #include "wine/heap.h"
 #include "wine/debug.h"
+#include "wine/exception.h"
 #include "wine/unicode.h"
 
 #include "msipriv.h"
@@ -1207,6 +1208,7 @@ static enum platform parse_platform( const WCHAR *str )
     else if (!strcmpW( str, szIntel64 )) return PLATFORM_INTEL64;
     else if (!strcmpW( str, szX64 ) || !strcmpW( str, szAMD64 )) return PLATFORM_X64;
     else if (!strcmpW( str, szARM )) return PLATFORM_ARM;
+    else if (!strcmpW( str, szARM64 )) return PLATFORM_ARM64;
     return PLATFORM_UNKNOWN;
 }
 
@@ -1288,6 +1290,10 @@ static UINT validate_package( MSIPACKAGE *package )
 #ifndef __arm__
     if (package->platform == PLATFORM_ARM)
         return ERROR_INSTALL_PLATFORM_UNSUPPORTED;
+#endif
+#ifndef __aarch64__
+    if (package->platform == PLATFORM_ARM64)
+        return ERROR_INSTALL_PLATFORM_UNSUPPORTED;
 #endif
     if (package->platform == PLATFORM_X64)
     {
@@ -1465,18 +1471,8 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, DWORD dwOptions, MSIPACKAGE **pPackage)
     if( szPackage[0] == '#' )
     {
         handle = atoiW(&szPackage[1]);
-        db = msihandle2msiinfo( handle, MSIHANDLETYPE_DATABASE );
-        if( !db )
-        {
-            MSIHANDLE remote;
-
-            if (!(remote = msi_get_remote(handle)))
-                return ERROR_INVALID_HANDLE;
-
-            WARN("MsiOpenPackage not allowed during a custom action!\n");
-
-            return ERROR_FUNCTION_FAILED;
-        }
+        if (!(db = msihandle2msiinfo(handle, MSIHANDLETYPE_DATABASE)))
+            return ERROR_INVALID_HANDLE;
     }
     else
     {
@@ -1624,6 +1620,13 @@ UINT MSI_OpenPackageW(LPCWSTR szPackage, DWORD dwOptions, MSIPACKAGE **pPackage)
         package->log_file = CreateFileW( gszLogFile, GENERIC_WRITE, FILE_SHARE_WRITE, NULL,
                                          OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
 
+    if (!msi_init_assembly_caches( package ))
+    {
+        ERR("can't initialize assembly caches\n");
+        msiobj_release( &package->hdr );
+        return ERROR_FUNCTION_FAILED;
+    }
+
     /* FIXME: when should these messages be sent? */
     data_row = MSI_CreateRecord(3);
     if (!data_row)
@@ -1739,8 +1742,16 @@ MSIHANDLE WINAPI MsiGetActiveDatabase(MSIHANDLE hInstall)
     }
     else if ((remote = msi_get_remote(hInstall)))
     {
-        handle = remote_GetActiveDatabase(remote);
-        handle = alloc_msi_remote_handle(handle);
+        __TRY
+        {
+            handle = remote_GetActiveDatabase(remote);
+            handle = alloc_msi_remote_handle(handle);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            handle = 0;
+        }
+        __ENDTRY
     }
 
     return handle;
@@ -2100,7 +2111,15 @@ INT WINAPI MsiProcessMessage( MSIHANDLE hInstall, INSTALLMESSAGE eMessageType,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        ret = remote_ProcessMessage(remote, eMessageType, (struct wire_record *)&record->count);
+        __TRY
+        {
+            ret = remote_ProcessMessage(remote, eMessageType, (struct wire_record *)&record->count);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
 
         msiobj_release(&record->hdr);
         return ret;
@@ -2108,6 +2127,7 @@ INT WINAPI MsiProcessMessage( MSIHANDLE hInstall, INSTALLMESSAGE eMessageType,
 
     ret = MSI_ProcessMessage( package, eMessageType, record );
 
+    msiobj_release( &record->hdr );
     msiobj_release( &package->hdr );
     return ret;
 }
@@ -2136,22 +2156,14 @@ end:
     return r;
 }
 
-void msi_reset_folders( MSIPACKAGE *package, BOOL source )
+void msi_reset_source_folders( MSIPACKAGE *package )
 {
     MSIFOLDER *folder;
 
     LIST_FOR_EACH_ENTRY( folder, &package->folders, MSIFOLDER, entry )
     {
-        if ( source )
-        {
-            msi_free( folder->ResolvedSource );
-            folder->ResolvedSource = NULL;
-        }
-        else
-        {
-            msi_free( folder->ResolvedTarget );
-            folder->ResolvedTarget = NULL;
-        }
+        msi_free( folder->ResolvedSource );
+        folder->ResolvedSource = NULL;
     }
 }
 
@@ -2230,12 +2242,22 @@ UINT WINAPI MsiSetPropertyW( MSIHANDLE hInstall, LPCWSTR szName, LPCWSTR szValue
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_SetProperty(remote, szName, szValue);
+        __TRY
+        {
+            ret = remote_SetProperty(remote, szName, szValue);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return ret;
     }
 
     ret = msi_set_property( package->db, szName, szValue, -1 );
     if (ret == ERROR_SUCCESS && !strcmpW( szName, szSourceDir ))
-        msi_reset_folders( package, TRUE );
+        msi_reset_source_folders( package );
 
     msiobj_release( &package->hdr );
     return ret;
@@ -2407,7 +2429,16 @@ UINT WINAPI MsiGetPropertyA(MSIHANDLE hinst, const char *name, char *buf, DWORD
             return ERROR_INVALID_HANDLE;
         }
 
-        r = remote_GetProperty(remote, nameW, &value, &len);
+        __TRY
+        {
+            r = remote_GetProperty(remote, nameW, &value, &len);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
         heap_free(nameW);
 
         if (!r)
@@ -2463,7 +2494,16 @@ UINT WINAPI MsiGetPropertyW(MSIHANDLE hinst, const WCHAR *name, WCHAR *buf, DWOR
         if (!(remote = msi_get_remote(hinst)))
             return ERROR_INVALID_HANDLE;
 
-        r = remote_GetProperty(remote, name, &value, &len);
+        __TRY
+        {
+            r = remote_GetProperty(remote, name, &value, &len);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
         if (!r)
         {
             /* String might contain embedded nulls.
@@ -2644,7 +2684,10 @@ UINT __cdecl s_remote_FormatRecord(MSIHANDLE hinst, struct wire_record *remote_r
     {
         *value = midl_user_allocate(++size * sizeof(WCHAR));
         if (!*value)
+        {
+            MsiCloseHandle(rec);
             return ERROR_OUTOFMEMORY;
+        }
         r = MsiFormatRecordW(hinst, rec, *value, &size);
     }
 
