@@ -98,6 +98,7 @@ static UINT OpenSourceKey(LPCWSTR szProduct, HKEY* key, DWORD dwOptions,
         if (rc != ERROR_SUCCESS)
             rc = ERROR_BAD_CONFIGURATION;
     }
+    RegCloseKey(rootkey);
 
     return rc;
 }
@@ -591,8 +592,11 @@ UINT WINAPI MsiSourceListGetInfoW( LPCWSTR szProduct, LPCWSTR szUserSid,
                               0, 0, NULL, &size);
         if (rc != ERROR_SUCCESS)
         {
-            RegCloseKey(sourcekey);
-            return ERROR_SUCCESS;
+            static WCHAR szEmpty[1] = { '\0' };
+            rc = ERROR_SUCCESS;
+            source = NULL;
+            ptr = szEmpty;
+            goto output_out;
         }
 
         source = msi_alloc(size);
@@ -626,7 +630,7 @@ UINT WINAPI MsiSourceListGetInfoW( LPCWSTR szProduct, LPCWSTR szUserSid,
             else
                 ptr++;
         }
-
+output_out:
         if (szValue)
         {
             if (strlenW(ptr) < *pcchValue)
@@ -1006,7 +1010,7 @@ static UINT fill_source_list(struct list *sourcelist, HKEY sourcekey, DWORD *cou
 
     while (r == ERROR_SUCCESS)
     {
-        size = sizeof(name) / sizeof(name[0]);
+        size = ARRAY_SIZE(name);
         r = RegEnumValueW(sourcekey, index, name, &size, NULL, NULL, NULL, &val_size);
         if (r != ERROR_SUCCESS)
             return r;
@@ -1326,3 +1330,21 @@ UINT WINAPI MsiSourceListClearSourceW(LPCWSTR szProductCodeOrPatchCode, LPCWSTR
           dwContext, dwOptions, debugstr_w(szSource));
     return ERROR_SUCCESS;
 }
+
+/******************************************************************
+ *  MsiSourceListForceResolutionA (MSI.@)
+ */
+UINT WINAPI MsiSourceListForceResolutionA(const CHAR *product, const CHAR *user, DWORD reserved)
+{
+    FIXME("(%s %s %x)\n", debugstr_a(product), debugstr_a(user), reserved);
+    return ERROR_SUCCESS;
+}
+
+/******************************************************************
+ *  MsiSourceListForceResolutionW (MSI.@)
+ */
+UINT WINAPI MsiSourceListForceResolutionW(const WCHAR *product, const WCHAR *user, DWORD reserved)
+{
+    FIXME("(%s %s %x)\n", debugstr_w(product), debugstr_w(user), reserved);
+    return ERROR_SUCCESS;
+}
