@@ -142,10 +142,24 @@ struct UNWIND_INFO
  *
  * Get the trap code for a signal.
  */
-static inline enum arm_trap_code get_trap_code( const ucontext_t *sigcontext )
+static inline enum arm_trap_code get_trap_code( int signal, const ucontext_t *sigcontext )
 {
 #ifdef TRAP_sig
-    return TRAP_sig(sigcontext);
+    enum arm_trap_code trap = TRAP_sig(sigcontext);
+    if (trap)
+        return trap;
+    /* trap is 0 on arm64 kernel */
+    switch (signal)
+    {
+    case SIGILL:
+        return TRAP_ARM_PRIVINFLT;
+    case SIGSEGV:
+        return TRAP_ARM_PAGEFLT;
+    case SIGBUS:
+        return TRAP_ARM_ALIGNFLT;
+    default:
+        return trap;
+    }
 #else
     return TRAP_ARM_UNKNOWN;  /* unknown trap code */
 #endif
@@ -283,9 +297,12 @@ __ASM_STDCALL_FUNC( RtlCaptureContext, 4,
 void DECLSPEC_HIDDEN set_cpu_context( const CONTEXT *context );
 __ASM_GLOBAL_FUNC( set_cpu_context,
                    ".arm\n\t"
-                   "ldr r1, [r0, #0x44]\n\t"  /* context->Cpsr */
-                   "msr CPSR_f, r1\n\t"
+                   "ldr r2, [r0, #0x44]\n\t"  /* context->Cpsr */
+                   "tst r2, #0x20\n\t"        /* thumb? */
                    "ldr r1, [r0, #0x40]\n\t"  /* context->Pc */
+                   "orrne r1, r1, #1\n\t"     /* Adjust PC according to thumb */
+                   "biceq r1, r1, #1\n\t"     /* Adjust PC according to arm */
+                   "msr CPSR_f, r2\n\t"
                    "ldr lr, [r0, #0x3c]\n\t"  /* context->Lr */
                    "ldr sp, [r0, #0x38]\n\t"  /* context->Sp */
                    "push {r1}\n\t"
@@ -715,7 +732,7 @@ static void segv_handler( int signal, siginfo_t *info, void *ucontext )
     ucontext_t *context = ucontext;
 
     /* check for page fault inside the thread stack */
-    if (get_trap_code(context) == TRAP_ARM_PAGEFLT &&
+    if (get_trap_code(signal, context) == TRAP_ARM_PAGEFLT &&
         (char *)info->si_addr >= (char *)NtCurrentTeb()->DeallocationStack &&
         (char *)info->si_addr < (char *)NtCurrentTeb()->Tib.StackBase &&
         virtual_handle_stack_fault( info->si_addr ))
@@ -732,7 +749,7 @@ static void segv_handler( int signal, siginfo_t *info, void *ucontext )
     rec = setup_exception( context, raise_segv_exception );
     if (rec->ExceptionCode == EXCEPTION_STACK_OVERFLOW) return;
 
-    switch(get_trap_code(context))
+    switch(get_trap_code(signal, context))
     {
     case TRAP_ARM_PRIVINFLT:   /* Invalid opcode exception */
         rec->ExceptionCode = EXCEPTION_ILLEGAL_INSTRUCTION;
@@ -753,7 +770,7 @@ static void segv_handler( int signal, siginfo_t *info, void *ucontext )
         rec->ExceptionInformation[1] = 0xffffffff;
         break;
     default:
-        ERR("Got unexpected trap %d\n", get_trap_code(context));
+        ERR("Got unexpected trap %d\n", get_trap_code(signal, context));
         rec->ExceptionCode = EXCEPTION_ILLEGAL_INSTRUCTION;
         break;
     }
@@ -1058,6 +1075,16 @@ BOOLEAN CDECL RtlAddFunctionTable( RUNTIME_FUNCTION *table, DWORD count, DWORD a
     return TRUE;
 }
 
+/**********************************************************************
+ *              RtlInstallFunctionTableCallback   (NTDLL.@)
+ */
+BOOLEAN CDECL RtlInstallFunctionTableCallback( DWORD table, DWORD base, DWORD length,
+                                               PGET_RUNTIME_FUNCTION_CALLBACK callback, PVOID context, PCWSTR dll )
+{
+    FIXME( "%x %x %d %p %p %s: stub\n", table, base, length, callback, context, wine_dbgstr_w(dll) );
+    return TRUE;
+}
+
 /*************************************************************************
  *              RtlAddGrowableFunctionTable   (NTDLL.@)
  */
