@@ -21,14 +21,22 @@
 #ifndef __WINE_NTOSKRNL_PRIVATE_H
 #define __WINE_NTOSKRNL_PRIVATE_H
 
-struct _OBJECT_TYPE {
+struct _OBJECT_TYPE
+{
     const WCHAR *name;            /* object type name used for type validation */
     void *(*constructor)(HANDLE); /* used for creating an object from server handle */
     void (*release)(void*);       /* called when the last reference is released */
 };
 
+struct _KTHREAD
+{
+    DISPATCHER_HEADER header;
+    CLIENT_ID id;
+};
+
 void *alloc_kernel_object( POBJECT_TYPE type, HANDLE handle, SIZE_T size, LONG ref ) DECLSPEC_HIDDEN;
 HANDLE kernel_object_handle( void *obj, unsigned int access ) DECLSPEC_HIDDEN;
+NTSTATUS kernel_object_from_handle( HANDLE handle, POBJECT_TYPE type, void **ret ) DECLSPEC_HIDDEN;
 
 extern POBJECT_TYPE ExEventObjectType;
 extern POBJECT_TYPE ExSemaphoreObjectType;
