@@ -22,13 +22,10 @@
 
 #include "wine/test.h"
 #include "winbase.h"
-#include "initguid.h"
 #include "dshow.h"
 #include "winternl.h"
-
-#include "fil_data.h"
-
-DEFINE_GUID(GUID_NULL,0,0,0,0,0,0,0,0,0,0,0);
+#include "initguid.h"
+#include "wine/fil_data.h"
 
 /* Helper function, checks if filter with given name was enumerated. */
 static BOOL enum_find_filter(const WCHAR *wszFilterName, IEnumMoniker *pEnum)
@@ -241,7 +238,7 @@ static void test_legacy_filter_registration(void)
     /* Register* functions need a filter class key to write pin and pin media
      * type data to. Create a bogus class key for it. */
     CoCreateGuid(&clsid);
-    StringFromGUID2(&clsid, clsidstring, sizeof(clsidstring));
+    StringFromGUID2(&clsid, clsidstring, ARRAY_SIZE(clsidstring));
     lstrcpyW(key_name, clsidW);
     lstrcatW(key_name, clsidstring);
     ret = RegCreateKeyExW(HKEY_CLASSES_ROOT, key_name, 0, NULL, 0, KEY_WRITE, NULL, &hkey, NULL);
