@@ -24,7 +24,6 @@
  */
  
 #include <stdarg.h>
-#include <stdio.h>
 #include <string.h>
 #include <assert.h>
 
@@ -37,7 +36,6 @@
 
 #include "wine/heap.h"
 #include "main.h"
-#include "regproc.h"
 
 /* spaces dividing hex and ASCII */
 #define DIV_SPACES 4
@@ -96,7 +94,7 @@ static LPWSTR HexEdit_GetLineText(int offset, BYTE *pData, LONG cbData, LONG pad
     for (i = 0; i < cbData; i++)
     {
         /* (C1_ALPHA|C1_BLANK|C1_PUNCT|C1_DIGIT|C1_LOWER|C1_UPPER) */
-        if (isprint(pData[offset + i]))
+        if (iswprint(pData[offset + i]))
             lpszLine[6 + cbData * 3 + pad * 3 + DIV_SPACES + i] = pData[offset + i];
         else
             lpszLine[6 + cbData * 3 + pad * 3 + DIV_SPACES + i] = '.';
