@@ -127,7 +127,7 @@ struct device
     struct object          obj;           /* object header */
     struct device_manager *manager;       /* manager for this device (or NULL if deleted) */
     char                  *unix_path;     /* path to unix device if any */
-    client_ptr_t           user_ptr;      /* opaque ptr for client side */
+    struct list            kernel_object; /* list of kernel object pointers */
     struct list            entry;         /* entry in device manager list */
     struct list            files;         /* list of open files */
 };
@@ -137,6 +137,7 @@ static struct object_type *device_get_type( struct object *obj );
 static void device_destroy( struct object *obj );
 static struct object *device_open_file( struct object *obj, unsigned int access,
                                         unsigned int sharing, unsigned int options );
+static struct list *device_get_kernel_obj_list( struct object *obj );
 
 static const struct object_ops device_ops =
 {
@@ -156,7 +157,7 @@ static const struct object_ops device_ops =
     directory_link_name,              /* link_name */
     default_unlink_name,              /* unlink_name */
     device_open_file,                 /* open_file */
-    no_kernel_obj_list,               /* get_kernel_obj_list */
+    device_get_kernel_obj_list,       /* get_kernel_obj_list */
     no_close_handle,                  /* close_handle */
     device_destroy                    /* destroy */
 };
@@ -460,7 +461,7 @@ static struct object *device_open_file( struct object *obj, unsigned int access,
         params.create.access  = access;
         params.create.sharing = sharing;
         params.create.options = options;
-        params.create.device  = file->device->user_ptr;
+        params.create.device  = get_kernel_object_ptr( device->manager, &device->obj );
 
         if ((irp = create_irp( file, &params, NULL )))
         {
@@ -471,6 +472,12 @@ static struct object *device_open_file( struct object *obj, unsigned int access,
     return &file->obj;
 }
 
+static struct list *device_get_kernel_obj_list( struct object *obj )
+{
+    struct device *device = (struct device *)obj;
+    return &device->kernel_object;
+}
+
 static void device_file_dump( struct object *obj, int verbose )
 {
     struct device_file *file = (struct device_file *)obj;
@@ -614,20 +621,17 @@ static int device_file_ioctl( struct fd *fd, ioctl_code_t code, struct async *as
 }
 
 static struct device *create_device( struct object *root, const struct unicode_str *name,
-                                     struct device_manager *manager, unsigned int attr )
+                                     struct device_manager *manager )
 {
     struct device *device;
 
-    if ((device = create_named_object( root, &device_ops, name, attr, NULL )))
+    if ((device = create_named_object( root, &device_ops, name, 0, NULL )))
     {
-        if (get_error() != STATUS_OBJECT_NAME_EXISTS)
-        {
-            /* initialize it if it didn't already exist */
-            device->unix_path = NULL;
-            device->manager = manager;
-            list_add_tail( &manager->devices, &device->entry );
-            list_init( &device->files );
-        }
+        device->unix_path = NULL;
+        device->manager = manager;
+        list_add_tail( &manager->devices, &device->entry );
+        list_init( &device->kernel_object );
+        list_init( &device->files );
     }
     return device;
 }
@@ -641,6 +645,7 @@ struct object *create_unix_device( struct object *root, const struct unicode_str
     {
         device->unix_path = strdup( unix_path );
         device->manager = NULL;  /* no manager, requests go straight to the Unix device */
+        list_init( &device->kernel_object );
         list_init( &device->files );
     }
     return &device->obj;
@@ -793,10 +798,13 @@ DECL_HANDLER(create_device)
         return;
     }
 
-    if ((device = create_device( root, &name, manager, req->attributes )))
+    if ((device = create_device( root, &name, manager )))
     {
-        device->user_ptr = req->user_ptr;
-        reply->handle = alloc_handle( current->process, device, req->access, req->attributes );
+        struct kernel_object *ptr = set_kernel_object( manager, &device->obj, req->user_ptr );
+        if (ptr)
+            grab_kernel_object( ptr );
+        else
+            set_error( STATUS_NO_MEMORY );
         release_object( device );
     }
 
@@ -808,13 +816,23 @@ DECL_HANDLER(create_device)
 /* delete a device */
 DECL_HANDLER(delete_device)
 {
+    struct device_manager *manager;
+    struct kernel_object *ref;
     struct device *device;
 
-    if ((device = (struct device *)get_handle_obj( current->process, req->handle, 0, &device_ops )))
+    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
+                                                             0, &device_manager_ops )))
+        return;
+
+    if ((ref = kernel_object_from_ptr( manager, req->device )) && ref->object->ops == &device_ops)
     {
+        device = (struct device *)grab_object( ref->object );
         delete_device( device );
         release_object( device );
     }
+    else set_error( STATUS_INVALID_HANDLE );
+
+    release_object( manager );
 }
 
 
@@ -848,8 +866,8 @@ DECL_HANDLER(get_next_device_request)
         irp = LIST_ENTRY( ptr, struct irp_call, mgr_entry );
         if (irp->thread)
         {
-            reply->client_pid = get_process_id( irp->thread->process );
-            reply->client_tid = get_thread_id( irp->thread );
+            reply->client_thread = get_kernel_object_ptr( manager, &irp->thread->obj );
+            reply->client_tid    = get_thread_id( irp->thread );
         }
         reply->params = irp->params;
         iosb = irp->iosb;
